#!/bin/bash

function is_ostree_system() {
    local immutable_system=$(installer_get "DI_IMMUTABLE_SYSTEM")
    if [[ $immutable_system == "true" ]]; then
        return 0
    else
        return 1
    fi
}

function ostree_get_deploy() {
    local commitID ostree ostree_refspec sysroot
    sysroot=$1
    ostree_refspec=$2
    commitID=$(ostree --repo=$sysroot/ostree/repo log "$ostree_refspec" | grep commit | head -n 1 | awk '{print $2}')
    ostree="/ostree/deploy/deepin/deploy/${commitID}.0"
    echo "$ostree"
}

function ostree_sys_init() {
    #rsync -azAX /ostree /target/
    unsquashfs_file "$DI_LIVE_DIR_ENV/live/filesystem.squashfs" /target/ ostree
    mkdir -p /target/boot || true

    local sysroot=/target
    local osname=deepin
    local ostree_refspec=$DI_REPO_BASE_NAME_ENV

    sysCommit=$(ostree rev-parse --repo=$sysroot/ostree/repo $ostree_refspec)
    info "sysCommit=$sysCommit"

    mkdir -p $sysroot/ostree/deploy/$osname/deploy || true
    ostree checkout --repo=$sysroot/ostree/repo $ostree_refspec "$sysroot/ostree/deploy/$osname/deploy/$sysCommit.0" || error "ostree_sys_init failed"
    info "ostree_sys_init ok"
}

function ostree_ext_mount() {
    local lowerdir=/target$1
    local ostree=$2
    local bootdir=$3


    local datadir=/target/persistent
    local extDeploy=$(realpath $datadir/$ostree)
    #local currentDataDir=$(dirname "$extDeploy")
    [ ! -d $datadir/overlay/work ] && mkdir -p $datadir/overlay/work
    [ ! -d $datadir/overlay/merged ] && mkdir -p $datadir/overlay/merged

    local otDeploy=$datadir/overlay/merged

    info "lowerdir=$lowerdir"
    info "extDeploy=$extDeploy"
    info "otDeploy=$otDeploy"


    modprobe overlay
    mount -t overlay -o lowerdir="$lowerdir",upperdir="$extDeploy",workdir="$datadir/overlay/work" overlay $otDeploy || error "ostree_ext_mount failed"
    mount --make-private $otDeploy

    mkdir -p $otDeploy/etc
    if [ -d $otDeploy/usr/etc ]; then
        info "$otDeploy/usr/etc exists"
        mount -o bind,rw $otDeploy/usr/etc $otDeploy/etc || error "mount $otDeploy/usr/etc failed"
    else
        mount -o bind,rw $otDeploy/etc $otDeploy/etc || error "mount $otDeploy/etc failed"
    fi


    mount -o bind $datadir $otDeploy/persistent || error "mount $datadir/persistent failed"
    mount -o bind /target $otDeploy/sysroot || error "mount $datadir/sysroot failed"
    mount --bind -o remount,ro $otDeploy/sysroot $otDeploy/sysroot || error "mount --bind -o remount,ro $otDeploy/sysroot $otDeploy/sysroot failed"
    mkdir -p $datadir/ostree/deploy/deepin/var
    mount --bind $datadir/ostree/deploy/deepin/var $otDeploy/var || error "mount $datadir/var failed"
   

    local data_commitid_idx=$(echo $extDeploy | awk -F/ '{print $(NF-1)}')
    local data_overlay_root=/target/persistent/overlay/data/$data_commitid_idx
    info "data_commitid_idx=$data_commitid_idx"
    mkdir -p $data_overlay_root

    # mount opt overlay
    mkdir -p $data_overlay_root/opt-upper $data_overlay_root/opt-work
    if [ -d $otDeploy/usr/opt ]; then
        info "$otDeploy/usr/opt exists"
        mkdir -p $extDeploy/usr/opt
        mount -t overlay -o rw,relatime,lowerdir=$extDeploy/usr/opt:$lowerdir/usr/opt,upperdir=$data_overlay_root/opt-upper,workdir=$data_overlay_root/opt-work opt-overlay $otDeploy/opt || error "mount overlay $otDeploy/usr/opt failed"
    else
        mkdir -p $extDeploy/opt
        mount -t overlay -o rw,relatime,lowerdir=$extDeploy/opt:$lowerdir/opt,upperdir=$data_overlay_root/opt-upper,workdir=$data_overlay_root/opt-work opt-overlay $otDeploy/opt || error "mount overlay $otDeploy/usr/opt failed"
    fi
   
    # mount usr overlay
    mkdir -p $data_overlay_root/usr-upper $data_overlay_root/usr-work
    mount -t overlay -o rw,relatime,lowerdir=$otDeploy/usr,upperdir=$data_overlay_root/usr-upper,workdir=$data_overlay_root/usr-work usr-overlay $otDeploy/usr || error "mount overlay $otDeploy/usr failed"
   
    # mount etc overlay
    mkdir -p $data_overlay_root/etc-upper $data_overlay_root/etc-work
    mount -t overlay -o rw,relatime,lowerdir=$otDeploy/etc,upperdir=$data_overlay_root/etc-upper,workdir=$data_overlay_root/etc-work etc-overlay $otDeploy/etc || error "mount overlay $otDeploy/etc failed"

    if [ -n "$bootdir" ]; then
        mount -o bind,rw $bootdir $otDeploy/boot
    fi

    mount --make-private /target
    mount --move $otDeploy /target
}

function mount_partitions() {
    local mount_points
    mount_points=$(installer_get "DI_MOUNTS_POINTS")
    #mount_points="/dev/nvme0n1p3=/;/dev/nvme0n1p2=/boot;/dev/nvme0n1p1=/boot/efi"
    mount_points=(${mount_points//;/ })
    declare -A mount_map
    for p in "${mount_points[@]}"
    do
        kv=(${p//=/ })
        if [[ "${kv[1]}" != "swap" ]];then
            mount_map["${kv[1]}"]="${kv[0]}"
        fi
    done

    keys=("${!mount_map[@]}")
    sorted_keys=($(printf "%s\n" "${keys[@]}" | sort))
    for key in "${sorted_keys[@]}"; do
        local part="${mount_map[$key]}"
        if ! findmnt $part; then
            mkdir -p "/target/$key" || error "Failed to create mount point /target/$key"
            mount "${mount_map[$key]}" "/target/$key" || error "Failed to mount ${mount_map[$key]} /target/$key"
            echo "mount ${mount_map[$key]} /target$key"
        fi
    done
}
